package server.command;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import common.collect.Filter;
import common.collect.Lists;
import common.collect.Sets;
import server.network.Player;

public class PlayerListParser extends PlayerParser {
	public PlayerListParser(String name, boolean useSender, UserPolicy policy) {
		super(name, useSender, policy);
	}

	public Object parse(CommandEnvironment env, String input) {
		if(input.equals("*")) {
			if(env.getServer().getPlayers().isEmpty())
				throw new RunException("Keine Spieler gefunden");
			Collection<Player> filtered = Filter.filter(env.getServer().getPlayers(), player -> this.policy.applies(env, env.getExecutor(), player));
			if(filtered.isEmpty())
				throw new RunException("Keine erlaubten Spieler gefunden");
			return Lists.newArrayList(filtered);
		}
		Set<Player> set = Sets.newHashSet();
		for(String tok : input.split(",", -1)) {
			set.add((Player)super.parse(env, tok));
		}
		if(set.isEmpty())
			throw new RunException("Keine Spieler gefunden");
		return Lists.newArrayList(set);
	}
	
	public Object getDefault(CommandEnvironment env) {
		Player net = (Player)super.getDefault(env);
		return net == null ? null : Lists.newArrayList(net);
	}
	
	public Collection<String> getCompletions(CommandEnvironment env) {
		Collection<String> comp = super.getCompletions(env);
		comp.add("*");
		return comp;
	}

	public Class<?> getTypeClass(boolean required) {
		return List.class;
	}
}
