package server.command.commands;

import java.util.Collection;
import java.util.Map.Entry;

import common.collect.Lists;
import common.color.TextColor;
import common.init.Config;
import common.init.Config.ValueType;
import server.command.ArgumentSplitter;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.command.StringCompleter;

public class CommandSv extends Command {
	public CommandSv() {
		super("sv");

		this.setParamsOptional();
		this.addString("variable", false, new StringCompleter() {
			public Collection<String> complete(CommandEnvironment env) {
				return Config.VARS.keySet();
			}
		});
		this.addString("value", true);
	}

	public Iterable<String> getCustomCompletions(CommandEnvironment env, Executor exec, ArgumentSplitter parsed, String param, String parser, String arg) {
		if(param.equals("value") && parsed.has("variable")) {
			Config.Value sv = Config.VARS.get(parsed.get("variable"));
			if(sv != null && sv.type == ValueType.BOOLEAN)
				return Boolean.parseBoolean(sv.getValue()) ? Lists.newArrayList("false", "true") : Lists.newArrayList("true", "false");
			else if(sv != null && !sv.noDef && (sv.type == ValueType.INTEGER || sv.type == ValueType.FLOAT))
				return Lists.newArrayList(sv.def);
		}
		return null;
	}
	
	private String formatVariable(String name, Config.Value sv, String separator, boolean censor) {
		String value = sv.getValue();
		StringBuilder sb = new StringBuilder(TextColor.YELLOW + name + TextColor.GRAY + " " + separator + " ");
		if(sv.noDef && sv.def.equals(value))
			sb.append(TextColor.DGRAY + "[ - ]");
		else if(censor && name.equals("password") && !value.isEmpty())
			sb.append(TextColor.NEON + "'****'");
		else if(sv.type == ValueType.STRING)
			sb.append(TextColor.NEON + "'" + value + "'");
		else
			sb.append(((sv.type == ValueType.BOOLEAN ? (value.equals("true") ? TextColor.GREEN : TextColor.RED) : TextColor.BLUE)) + value);
		if(!sv.def.equals(value))
			sb.append(TextColor.GRAY + " (" + (sv.noDef ? TextColor.DGRAY + "[ - ]" : TextColor.BROWN + sv.def) + TextColor.GRAY + ")");
		return sb.toString();
	}
	
	public Object exec(CommandEnvironment env, Executor exec, String variable, String value) {
		if(variable == null) {
			if(value != null)
				throw new RunException("Kann keinen Wert ohne eine Variable angeben");
			for(Entry<String, Config.Value> entry : Config.VARS.entrySet()) {
				exec.logConsole(this.formatVariable(entry.getKey(), entry.getValue(), "=", true));
			}
			exec.logConsole(TextColor.GREEN + "SVARs insgesamt registriert: %d", Config.VARS.size());
			return null;
		}
		Config.Value sv = Config.VARS.get(variable);
		if(sv == null)
			throw new RunException("Variable '%s' existiert nicht", variable);
		if(value != null) {
			if(sv.type == ValueType.BOOLEAN && !"true".equals(value) && !"false".equals(value)) {
				if(value.equals("0") || value.equals("1"))
					value = "" + value.equals("1");
				else if(value.equalsIgnoreCase("true") || value.equalsIgnoreCase("false"))
					value = value.toLowerCase();
				else
					throw new RunException("'%s' ist nicht 'true' (1) oder 'false' (0)", value);
			}
			else if(sv.type == ValueType.INTEGER) {
				try {
					Integer.parseInt(value);
				}
				catch(NumberFormatException e) {
					throw new RunException(TextColor.DRED + "'%s' ist keine gültige Zahl", value);
				}
			}
			else if(sv.type == ValueType.FLOAT) {
				try {
					Float.parseFloat(value);
				}
				catch(NumberFormatException e) {
					throw new RunException(TextColor.DRED + "'%s' ist keine gültige Zahl", value);
				}
			}
			Config.set(variable, value, true);
		}
		exec.logConsole(this.formatVariable(variable, sv, value == null ? "=" : "->", false));
		return sv.noDef && sv.def.equals(sv.getValue()) ? null : sv.getValue();
	}
}
